.wy-nav-content {
   max-width: 100vw;
}

/* override table width restrictions */
@media screen{

   .wy-table-responsive table td {
      /* !important prevents the common CSS stylesheets from overriding
         this as on RTD they are loaded after this stylesheet */
      white-space: normal !important;
      word-break: break-word;
   }

   .wy-table-responsive {
      overflow: visible !important;
   }
}
